/**	@file libmil1553ud.c
  *	@brief  
  *
  *	
  */

#include "stdafx.h"
#include "libmil1553ud.h"

/// \remark	   DMA
UINT32 mil1553UdGetCountDmaBlocksReadyRead(HANDLE dev, UINT8 channel){
	DWORD size;	//   ioctl 
	DMA_COUNT_REQUEST inputBuf;	//    

	inputBuf.count = 0;
	inputBuf.channel = channel;
	DeviceIoControl(dev, IOCTL_GET_NBLOCKS_RAW_DMA, &inputBuf, sizeof(DMA_COUNT_REQUEST), &inputBuf, sizeof(DMA_COUNT_REQUEST), &size, NULL);

	return inputBuf.count;
}

/// \remark	      
UINT32 mil1553UdGetCountInterruptBlocksReadyRead(HANDLE dev, UINT8 channel){
	UINT8 chn;	//,  
	UINT32 blocks;	// ,  
	DWORD size; //   ioctl 

	chn = channel;
	blocks = 0;
	DeviceIoControl(dev, IOCTL_GET_COUNT_INTERRUPT_BLOCKS, &chn, sizeof(UINT32), &blocks, sizeof(UINT32), &size, NULL);
	return blocks;
}

/// \remark	     
UINT32 mil1533udReadInterruptBlocks(HANDLE dev, UINT8 channel, UINT32 count, BlockInfo** pblock_info){
	UINT32 left, done = 0;
	DWORD size; //   ioctl 
	INTERRUPT_BLOCK_BUFFER *intBuf, *outBuf;
	intBuf = (INTERRUPT_BLOCK_BUFFER*) malloc(sizeof(INTERRUPT_BLOCK_BUFFER));
	outBuf = (INTERRUPT_BLOCK_BUFFER*) malloc(sizeof(INTERRUPT_BLOCK_BUFFER));

	*pblock_info = (BlockInfo*)malloc(sizeof(BlockInfo) * count); //   
	left = count;

	while(left > 0){	//     
		intBuf->channel = channel;
		intBuf->count_blocks = (left > INTERRUPT_BLOCKS_MAX) ? INTERRUPT_BLOCKS_MAX : left;
		left -= intBuf->count_blocks;
		DeviceIoControl(dev, IOCTL_READ_INTERRUPT_BLOCKS, intBuf, sizeof(INTERRUPT_BLOCK_BUFFER), outBuf, sizeof(INTERRUPT_BLOCK_BUFFER), &size, NULL);

		memcpy(*pblock_info + (done * sizeof(BlockInfo)), outBuf->blocks, outBuf->count_blocks * sizeof(BlockInfo));	//      
		done += outBuf->count_blocks;
	}

	free(intBuf);
	free(outBuf);
	return done;
}

/// \remark	 BlockInfo
void mil1533udFreeBlockInfo(BlockInfo* pblock_info){
	free(pblock_info);
}

/// \remark	    dma
UINT32 mil1533udReadDmaBlocks(HANDLE dev, UINT8 channel, UINT32 count, UINT8** dmaBuffer){
	UINT32 left, done=0;
	DWORD size; //   ioctl 
	DMA_READ_BLOCK* dmaBuf;
	dmaBuf = (DMA_READ_BLOCK*) malloc(sizeof(DMA_READ_BLOCK));

	*dmaBuffer = (UINT8*) malloc( sizeof(UINT8) * count * DMA_RAW_BLOCK_SIZE );	// 
	left = count;

	while(left > 0){	//     
		dmaBuf->channel = channel;
		dmaBuf->countBlocks = (left > DMA_BLOCKS_COUNT) ? DMA_BLOCKS_COUNT : left;
		left -= dmaBuf->countBlocks;

		DeviceIoControl(dev, IOCTL_READ_BLOCKS_RAW_DMA, dmaBuf, sizeof(DMA_READ_BLOCK), dmaBuf, sizeof(DMA_READ_BLOCK), &size, NULL);
		memcpy(*dmaBuffer + (done * DMA_RAW_BLOCK_SIZE), dmaBuf->data, dmaBuf->countBlocks * DMA_RAW_BLOCK_SIZE);	//      
		done += dmaBuf->countBlocks;
	}

	free(dmaBuf);
	return done;
}

/// \remark   DMA

UINT32 mil1553UdDmaBlockIsSuccessTransaction(UINT32 typeDmaBlock, UINT8 *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // 
        UINT32* dword = (UINT32*)(dmaBlock + 0 * sizeof(UINT32));
		return ((*dword >> 28) & 0x1) ? MIL1553UD_TRUE : MIL1553UD_FALSE;
    } else {    // 
        UINT32* dword = (UINT32*)(dmaBlock + 0 * sizeof(UINT32));
        return ((*dword >> 26) & 0x1) ? MIL1553UD_FALSE : MIL1553UD_TRUE;
    }
}

UINT32 mil1553UdDmaBlockGetTypeTransaction(UINT32 typeDmaBlock, UINT8 *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // 
        UINT32* dword = (UINT32*)(dmaBlock + 0 * sizeof(UINT32));
        return (*dword >> 1) & 0xF;
    } else {    // 
        UINT32* dword = (UINT32*)(dmaBlock + 0 * sizeof(UINT32));
        return (*dword >> 27) & 0xF;
    }
}

UINT32 mil1553UdDmaBlockGetFreeTimer(UINT32 typeDmaBlock, UINT8 *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // 
        UINT32* dword = (UINT32*)(dmaBlock + 1 * sizeof(UINT32));
        return *dword;
    } else {    // 
        UINT32* dword = (UINT32*)(dmaBlock + 4 * sizeof(UINT32));
        return *dword;
    }
}

UINT32 mil1553UdDmaBlockGetCountServiceWords16(UINT32 typeDmaBlock, UINT8 *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // 
        UINT32* dword = (UINT32*)(dmaBlock + 0 * sizeof(UINT32));
        return (*dword >> 11) & 0xF;
    } else {    // 
        UINT32* dword = (UINT32*)(dmaBlock + 0 * sizeof(UINT32));
        return (*dword >> 11) & 0xF;
    }
}

UINT32 mil1553UdDmaBlockGetCountDataWords16(UINT32 typeDmaBlock, UINT8 *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // 
        UINT32* dword = (UINT32*)(dmaBlock + 0 * sizeof(UINT32));
        return (*dword >> 5) & 0x3F;
    } else {    // 
        UINT32* dword = (UINT32*)(dmaBlock + 0 * sizeof(UINT32));
        return (*dword >> 5) & 0x3F;
    }
}

UINT32 mil1553UdDmaBlockTdModeSubaddress(UINT8 *dmaBlock)
{
    return *((UINT32*)dmaBlock) & 0x1F;
}

UINT32 mil1553UdDmaBlockGetActiveBus(UINT32 typeDmaBlock, UINT8 *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // 
        UINT32* dword = (UINT32*)(dmaBlock + 0 * sizeof(UINT32));
        return ( (*dword >> 15) & 0x1 ) ? MIL1553_BUS_B : MIL1553_BUS_A ;
    } else {    // 
        UINT32* dword = (UINT32*)(dmaBlock + 0 * sizeof(UINT32));
        return ( (*dword >> 31) & 0x1 ) ? MIL1553_BUS_B : MIL1553_BUS_A ;
    }
}

UINT8* mil1553UdDmaBlockGetPtrDataPart(UINT8 *dmaBlock)
{
    return dmaBlock + MIL_DMA_BLOCK_SIZE/2;
}

/// \remark   

UINT32 mil1553UdReadReg(HANDLE dev, UINT32 address, UINT8 channel){
	SADDR_DATA buf; // 
	DWORD size; //   ioctl 
	buf.daddr = address;
	buf.channel = channel;
	DeviceIoControl(dev, IOCTL_READ_REG, &buf, sizeof(SADDR_DATA), &buf, sizeof(SADDR_DATA), &size, NULL);
	return buf.data;
}

/// \remark   

void   mil1553UdWriteReg(HANDLE dev, UINT32 address, UINT8 channel, UINT32 data){
	SADDR_DATA buf; 
	DWORD size; //   ioctl 
	buf.daddr = address;
	buf.channel = channel;
	buf.data = data;
	DeviceIoControl(dev, IOCTL_WRITE_REG, &buf, sizeof(SADDR_DATA), NULL, 0, &size, NULL);
}

BC_RAM_BLOCK* mil1553UdNewBcRamBlock(UINT32 type, UINT32 sizeInDwords, UINT32 shiftInDwords)
{
    BC_RAM_BLOCK* block = (BC_RAM_BLOCK*)malloc(sizeof(BC_RAM_BLOCK));
    block->type = type;
    block->length = sizeInDwords;
    block->shift = shiftInDwords;
	return block;
}

void mil1553UdFreeBcRamBlock(BC_RAM_BLOCK *block)
{
    if (block != NULL) {
        free(block);
    }
}

BOOL mil1553UdBcProgram(HANDLE dev, UINT8 channel, UINT32 action)
{
    SADDR_DATA_BIT_MASK bitMask;    //   
	DWORD size; //   ioctl 

	bitMask.channel = channel;
    bitMask.daddr = MIL_REG(channel, BC_CONF_REG_PCI_CH);     //  
    bitMask.mask = 1 << 15; //   15  - BCSTRT
    if (action == BC_PROGRAM_START) {
        bitMask.data = 1 << 15; // BCSTRT  1
    } else {
        bitMask.data = 0;
    }

	return DeviceIoControl(dev, IOCTL_WRITE_REG_BIT_MASK, &bitMask, sizeof(SADDR_DATA_BIT_MASK), NULL, 0, &size, NULL);
}

/// \remark  
HANDLE mil1553UdOpen(WCHAR* devicePath){
	HANDLE hDevice = CreateFile(devicePath,
			GENERIC_READ | GENERIC_WRITE,
			FILE_SHARE_READ | FILE_SHARE_WRITE,
			NULL,
			OPEN_EXISTING,
			0,
			NULL);
	return hDevice;
}


/// \remark   
Mil1553UdDeviceList* mil1553UdEnumerateDevices(){
	HDEVINFO        hdi;
	DWORD	devIndex, size, wsize;
	SP_DEVICE_INTERFACE_DATA DeviceInterfaceData;
	PSP_DEVICE_INTERFACE_DETAIL_DATA pDeviceInterfaceDetail;
	BOOL status;
	SP_DEVINFO_DATA DevinfoData;
	HANDLE hDevice;
	TCHAR *DeviceName = NULL;

	Mil1553UdDeviceList* list = (Mil1553UdDeviceList*)malloc(sizeof(Mil1553UdDeviceList));	//    
	list->count = 0;

	hdi = SetupDiGetClassDevs(&GUID_DEVINTERFACE_MIL1553, NULL, NULL, DIGCF_DEVICEINTERFACE );	//     GUID_DEVINTERFACE_MIL1553
	DeviceInterfaceData.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);
	DevinfoData.cbSize = sizeof(SP_DEVINFO_DATA);
	while(SetupDiEnumDeviceInterfaces(hdi, NULL, &GUID_DEVINTERFACE_MIL1553, list->count++, &DeviceInterfaceData));	//  
	list->count--;

	if(list->count == 0)
		return list;

	list->device = (Mil1553UdDevice*)malloc(sizeof(Mil1553UdDevice) * list->count);	//    

	devIndex = 0;
	while(SetupDiEnumDeviceInterfaces(hdi, NULL, (LPGUID)&GUID_DEVINTERFACE_MIL1553, devIndex, &DeviceInterfaceData)){	//  
		list->device[devIndex].channelCount = 0;
		//      setupapi
		SetupDiGetDeviceInterfaceDetail(hdi, &DeviceInterfaceData, NULL, 0, &size, NULL);	
		pDeviceInterfaceDetail = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(size);
		if (!pDeviceInterfaceDetail) {
			devIndex++;
			continue;
        }

		pDeviceInterfaceDetail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
		status = SetupDiGetDeviceInterfaceDetail(hdi, &DeviceInterfaceData, pDeviceInterfaceDetail, size, NULL, &DevinfoData);
		if(!status){
			free(pDeviceInterfaceDetail);
			devIndex++;
			continue;
		}

		wsize = wcslen(pDeviceInterfaceDetail->DevicePath) + 1;
		list->device[devIndex].devicePath = (WCHAR*)malloc(sizeof(WCHAR) * wsize);
		wcsncpy_s(list->device[devIndex].devicePath, wsize, pDeviceInterfaceDetail->DevicePath, wsize);

		hDevice = mil1553UdOpen(pDeviceInterfaceDetail->DevicePath);
		if(hDevice == INVALID_HANDLE_VALUE){
			free(pDeviceInterfaceDetail);
			free(list->device[devIndex].devicePath);
			devIndex++;
			continue;
		}

		{
            SetupDiGetDeviceRegistryProperty(hdi, &DevinfoData, SPDRP_LOCATION_INFORMATION, NULL, (PBYTE)DeviceName, 0, &size);
            if (GetLastError() != ERROR_INSUFFICIENT_BUFFER) {
				free(list->device[devIndex].devicePath);
				devIndex++;
                continue;
            }

            DeviceName = (TCHAR*) malloc(size);
            if (!DeviceName) {
				free(list->device[devIndex].devicePath);
				devIndex++;
                continue;
            }

			//    
            status = SetupDiGetDeviceRegistryProperty(hdi, &DevinfoData, SPDRP_LOCATION_INFORMATION, NULL, (PBYTE)DeviceName, size, NULL);
            if(!status){
                free(DeviceName);
				free(list->device[devIndex].devicePath);
				devIndex++;
                continue;
            }

			//    
			list->device[devIndex].deviceLocation = (WCHAR*)malloc(sizeof(WCHAR) * wsize);
			wcsncpy_s(list->device[devIndex].deviceLocation, size, DeviceName, size);

            free(DeviceName);
        }

		//    ioctl
		VERSION version;	
		DeviceIoControl(hDevice, IOCTL_VERSION, NULL, 0, &version, sizeof(VERSION), &wsize, NULL);
		CloseHandle(hDevice);
		list->device[devIndex].deviceId = version.device_id;
		list->device[devIndex].vendorId = version.vendor_id;
		list->device[devIndex].channelCount = version.type;

		free(pDeviceInterfaceDetail);
		devIndex++;
	}

	SetupDiDestroyDeviceInfoList(hdi);
	return list;
}

/// \remark   
void mil1553UdFreeDeviceList(Mil1553UdDeviceList* list){
	for(UINT8 i=0;i<list->count;i++){
		if(list->device[i].channelCount > 0){
			free(list->device[i].devicePath);
			free(list->device[i].deviceLocation);
		}
	}
	free(list->device);
	free(list);
}