/**	@file
  *	@brief  .  ,    
  *
  *	
  */

#pragma comment(lib, "Setupapi.lib")

#ifndef __mil1553UdLIB_H__
#define __mil1553UdLIB_H__

#include <stdio.h>
#include <stdlib.h>
#include <Windows.h>
#include <SetupAPI.h>
#include <WinUser.h>
#include <devguid.h>
#include <malloc.h>

#include "mil1553ud.h"

#define MAXIMUM_FILENAME_LENGTH 256

#define MODE_RAW_IOCTL 1
#define MODE_BUS_CONTROLLER 2
#define MODE_REMOTE_TERMINAL 3

#define MIL1553UD_BC_DMA_BLOCK      0
#define MIL1553UD_RT_DMA_BLOCK      1

#define MIL1553UD_FALSE             0
#define MIL1553UD_TRUE              ~(MIL1553UD_FALSE)

#define MIL1553_BUS_A       0x0
#define MIL1553_BUS_B       ~MIL1553_BUS_A

#define MIL_DMA_BLOCK_SIZE 128

#define BC_PROGRAM_STOP     0x0
#define BC_PROGRAM_START    ~BC_PROGRAM_STOP

#define XEQ         0x1
#define XQF         0x15
#define XQG         0x16
#define XFG         0x1E
#define JMP         0x2
#define CAL         0x3
#define RTN         0x4
#define IRQ         0x6
#define HLT         0x7
#define DLY         0x8
#define WFT         0x9
#define CFT         0xA
#define CMT         0xB
#define LTT         0xD
#define LTH         0x18
#define LFT         0xE
#define SFT         0xF
#define WMP         0x1A
#define WMI         0x1C
#define DSZ         0x1D
#define FLG         0xC

#define IDENT       0xA

#define ALWAYS      0xF

#define INSTRUCTION(instruction, condition, parameter)   \
    (   \
    ( 0 << 31 ) | ( (instruction & 0x1F) << 26 ) | ( IDENT << 21 ) | ( (condition & 0x1F) << 16) | (parameter & 0xFFFF) \
    )

#define OPERATION_ODD_PART(command, timeout)    \
    (   \
    ( (command & 0xFFFF) << 16 ) | (timeout & 0xFFFF)  \
    )

#define OPERATION_EVEN_PART(cw1, parameter) \
    (   \
    ( (cw1 & 0xFFFF) << 16 ) | (parameter & 0xFFFF)  \
    )

#define COMMAND_WORD(adrOu, k, suba_mode, cntDw_code)  \
    (   \
    ( (adrOu & 0x1F) << 11 ) | ( (k & 0x1) << 10) | ( (suba_mode & 0x1F) << 5) | ( (cntDw_code & 0x1F) << 0)   \
    )

#define MIL_REG(numCh, RegName) \
    ( (numCh == CH_NUM_1) ? RegName##1 : ( (numCh == CH_NUM_2) ? RegName##2 : ( (numCh == CH_NUM_3) ? RegName##3 : RegName##4) ) )


typedef struct _mil1553Uddevice{
	WCHAR* devicePath;
	WCHAR* deviceLocation; 
	UINT32 vendorId;
	UINT32 deviceId;
	UINT8  channelCount;
} Mil1553UdDevice;

typedef struct _mil1553Uddevice_list{
	UINT8 count;
	Mil1553UdDevice* device;
} Mil1553UdDeviceList;

/**
* @brief   
*
* @param devicePath   
* @return    NULL,   
*/
HANDLE mil1553UdOpen(WCHAR* devicePath);

/**
* @brief     
*
* @return         
*/
Mil1553UdDeviceList* mil1553UdEnumerateDevices();

/**
* @brief     
*
*   ,   .
*
* @param list    
*/
void mil1553UdfreeDeviceList(Mil1553UdDeviceList* list);


/**
* @brief       DMA
*
* @param dev  
* @param channel    
* @return   DMA
*/
UINT32 mil1553UdGetCountDmaBlocksReadyRead(HANDLE dev, UINT8 channel);

/**
* @brief       
*
* @param dev  
* @param channel    
* @return   
*/
UINT32 mil1553UdGetCountInterruptBlocksReadyRead(HANDLE dev, UINT8 channel);


/**
* @brief   
*
* @param dev  
* @param channel    
* @param count  
* @param pblock_info   ,       BlockInfo,    
* @return   
*/
UINT32 mil1533udReadInterruptBlocks(HANDLE dev, UINT8 channel, UINT32 count, BlockInfo** pblock_info);

/**
* @brief    BlockInfo
*
* @param pblock_info    BlockInfo
*/
void mil1533udFreeBlockInfo(BlockInfo* pblock_info);


/**
* @brief   DMA
*
* @param dev  
* @param channel    
* @param count  
* @param dmaBuffer   ,        ,    
* @return   
*/
UINT32 mil1533udReadDmaBlocks(HANDLE dev, UINT8 channel, UINT32 count, UINT8** dmaBuffer);


/**
* @brief   DMA -   
*
* @param typeDmaBlock   DMA
* @param dmaBlock     
* @return MIL1553UDTRUE   ,  MIL1553UDFALSE
*/
UINT32 mil1553UdDmaBlockIsSuccessTransaction(UINT32 typeDmaBlock, UINT8 *dmaBlock);

/**
* @brief   DMA -   
*
* @param typeDmaBlock   DMA
* @param dmaBlock     
* @return MIL1553UD_BC_DMA_BLOCK  MIL1553UD_RT_DMA_BLOCK
*/
UINT32 mil1553UdDmaBlockGetTypeTransaction(UINT32 typeDmaBlock, UINT8 *dmaBlock);

/**
* @brief   DMA -   free 
*
* @param typeDmaBlock   DMA
* @param dmaBlock     
* @return  
*/
UINT32 mil1553UdDmaBlockGetFreeTimer(UINT32 typeDmaBlock, UINT8 *dmaBlock);

/**
* @brief   DMA -    
*
* @param typeDmaBlock   DMA
* @param dmaBlock     
* @return   
*/
UINT32 mil1553UdDmaBlockGetCountServiceWords16(UINT32 typeDmaBlock, UINT8 *dmaBlock);

/**
* @brief   DMA -    
*
* @param typeDmaBlock   DMA
* @param dmaBlock     
* @return   
*/
UINT32 mil1553UdDmaBlockGetCountDataWords16(UINT32 typeDmaBlock, UINT8 *dmaBlock);

/**
* @brief   DMA -    RT
*
* @param typeDmaBlock   DMA
* @param dmaBlock     
* @return 
*/
UINT32 mil1553UdDmaBlockTdModeSubaddress(UINT8 *dmaBlock);

/**
* @brief   DMA -   
*
* @param typeDmaBlock   DMA
* @param dmaBlock     
* @return MIL1553_BUS_A  MIL1553_BUS_B
*/
UINT32 mil1553UdDmaBlockGetActiveBus(UINT32 typeDmaBlock, UINT8 *dmaBlock);


/**
* @brief   DMA -    
*
* @param typeDmaBlock   DMA
* @param dmaBlock     
* @return      
*/
UINT8* mil1553UdDmaBlockGetPtrDataPart(UINT8 *dmaBlock);


/**
* @brief   
*
* @param dev  
* @param address 
* @param channel 
* @return  
*/
UINT32 mil1553UdReadReg(HANDLE dev, UINT32 address, UINT8 channel);

/**
* @brief   
*
* @param dev  
* @param address 
* @param channel 
* @param data   
*/
void   mil1553UdWriteReg(HANDLE dev, UINT32 address, UINT8 channel, UINT32 data);


/**
* @brief     BC_RAM_BLOCK
*
* @param type	 
* @param sizeInDwords	 
* @param shiftInDwords	   (   )
* @return   
*/
BC_RAM_BLOCK *mil1553UdNewBcRamBlock(UINT32 type, UINT32 sizeInDwords, UINT32 shiftInDwords);

/**
* @brief    BC_RAM_BLOCK
*
* @param block	   BC_RAM_BLOCK
*/
void mil1553UdFreeBcRamBlock(BC_RAM_BLOCK *block);


/**
* @brief       
*
* @param handle  
* @param channel 
* @param action  - BC_PROGRAM_STOP  BC_PROGRAM_START
* @return TRUE,     
*/
BOOL mil1553UdBcProgram(HANDLE dev, UINT8 channel, UINT32 action);

#endif